-----------------------------------------------------------------
----------------------- SOME COMMENTS BEFORE WE BEGIN -------------------
-----------------------------------------------------------------
-- This is a single line comment. Usually used at the end of lines 

--[[
   This is a multi line comment. It begins like the single line comment
   with two minus symbols, then two brackets.
   
   We usually use these if we want to describe something in more detail.
   Sometimes though, we are lazy and use the single line comments line
   after line after line, like I do in this example.
   
   To close the multi line comment, we end it the reverse way we began.
   ]]--
   
--[[
   This would be the normal header for an addon, where we type some gibberish 
   about the name of the addon, who made it etc.    
  
   Addon Test 1, by ins@ESOUI
 ]]--
-----------------------------------------------------------------
---------------------------- CODE BEGINS HERE ------------------- 
-----------------------------------------------------------------
--[[
 We initialize a table ({}) with the "code name" of the addon.
 Anything we reference to as "insADD1." with a dot will be put
 inside this table. We basically put all our code into this table.
 
 You can type /script d(insADD1) ingame to see how this works. 
 ]]-- 

-- Create a global table to hold the stuff for the addon.
if insADD1 == nil then insADD1 = {} end

-- Addon name is CASE SENSITIVE and MUST Match the file name (without the .lua part)
insADD1.name = "insaddontest1" -- this will not work
insADD1.name = "insAddonTest1" -- this will work

-- We include a version number
insADD1.version = 0.2

-- Setting up two local variables that can be accessed anywhere in the addon
-- with the colours we want to use. 
local insADD1cG = "|c00FF00"
local insADD1cW = "|cFFFFFF"

-- This is a global variable and it is dirty as it is "outside" the global table we created for our addon
insADD1var01 = "Global var insADD1var01 from insAddontest1.lua outside function"

-- This is a global variable and it is clean, since it is inside the global table for our addon
insADD1.ADD1varclean = "Global var insADD1var01 from insAddontest1.lua outside function, but inside our table"

-- This is a local variable
local insADD1var02 = "Local var insADD1var02 from insAddontest1.lua outside function"




--------------------------------------------------------------------
---  insADD1.table function
--       input: none
--       output: text dump of a table
-- 
-- info:
--   Outputs the contents of table "insADD1.vars" 
--   Then it will add some entries to it and print it again. 
--------------------------------------------------------------------
local function testtable()
    -- Print out the table as it is before we modify it.
    d(insADD1cW.."This is the table.")
    d(insADD1.vars)
    
    -- We set up 3 local variables we will insert into our table
    local ins1 = "Local variable ins1" -- a string
    local ins2 = 42  -- a number
    local ins3 = ins1..tostring(ins2)  -- a combination of the two
    
    -- We insert into our table (insADD1.vars)
    -- Using table.insert we inserted a line of text as a string.
    table.insert(insADD1.vars, "We insert a line of text")
    -- We then insert a variable+string combination. 
    table.insert(insADD1.vars, insADD1cG.."A coloured line of text")
    -- We then insert a string variable (plain text)
    table.insert(insADD1.vars, ins1)
    -- We then insert a number variable (42)
    table.insert(insADD1.vars, ins2)
    -- We then i  nsert a string variable, which was combined earlier from ins1 and ins2 local variables
    table.insert(insADD1.vars, ins3)
    
    d(insADD1cW.."This is the table after we had some fun.")
    d(insADD1.vars)
    
    d(insADD1cW.."We will now reference elements inside this table..")
    d(insADD1cG.."We will output the contents of (string):1.") 
    d(insADD1cG.."The reference: insADD1.vars[1]")
    d(insADD1.vars[1])
    d(insADD1cG.."We will output the contents of (table):key2.")
    d(insADD1cG.."The reference: insADD1.vars[\"key1\"]") -- notice we have to use \ in front of " to print it out.
    d(insADD1.vars["key1"])
    d(insADD1cG.."We will output the contents in (table):key2:(string):key2key1")
    d(insADD1cG.."The reference: insADD1.vars[\"key2\"][\"key2key1\"]") -- notice we have to use \ in front of " to print it out.
    d(insADD1.vars["key2"]["key2key1"])
    -- Notice how the table.inserts will automatically number upwards from 1.     
    -- Notice that tables with names (e.g. key1) need to be referenced enclosed in " 
    
end -- this is the end of insADD1.table function


--------------------------------------------------------------------
---  test function
--       input: none
--       output: text + variables will be set
-- 
-- info:
-- Typing "/addtest 1" ingame will call it. 
-- It sets two global and two local variables and prints out a line of text.
--------------------------------------------------------------------
local function test()
-- We set two local variables, these can only be accessed from within this function 
-- AFTER the function has been called
	local insADD1var100 = 100
	local insADD1var06 = "Local insADD1var06 from test() in insAddonTest1.lua"

-- We set two GLOBAL variables, they can be accessed from anywhere
-- AFTER the function has been called.
	insADD1var07 = "Global insADD1var07 from test() in insAddonTest1.lua"
-- This GLOBAL variable references the local variable insADD1var100
	insADD1var08 = "Global insADD1var08 from test() in insAddonTest1.lua - Referencing local insADD1var100:->"..tostring(insADD1var100)
-- This outputs a line of text to the user when the function is called.
	d("Simple test output from insAddonTest1.lua: test()")
end -- this is the end of insADD1.test function



--------------------------------------------------------------------
---  testvars function
--       input: none
--       output: text
-- 
-- info:
-- This function will display the variables and their source
-- Those with the output "Nil" we were unable to reach.
--------------------------------------------------------------------
local function testvars()
	local insADD1var05 = "insADD1var05 from testvars() in insAddonTest1.lua"
	local insadd1var10 = "insadd1var11 from testvars() in insAddonTest1.lua"
  d(insADD1cW.."Testing Variables function testvars in insAddonTest1.lua")
  d("1:(global outside functions): ")
  d(insADD1cG..tostring(insADD1var01))
	d("2:(local outside functions): ")
	d(insADD1cG..tostring(insADD1var02))
	d(insADD1cW.."The next variables are from inside functions")
	d("3:(local initialize): Local from another function == fail")
	d(insADD1cG..tostring(insADD1var03))
	d("4:(global initialize): Global from another function == ok")
	d(insADD1cG..tostring(insADD1var04))  
	d("5:(local insADD1.testvars): Local from this function == ok")
	d(insADD1cG..tostring(insADD1var05))
	d("6:(local insADD1.test): Local from another function == fail")
	d(insADD1cG..tostring(insADD1var06))
	d(insADD1cW.."This will only work if you run /testadd 1 first")
	d(insADD1cW.."Which will run insADD1.test() function. ")
	d("7:(global insADD1.test): Global from another function")
	d(insADD1cG..tostring(insADD1var07))
	d("8:(global insADD1.test): Global from another function, referencing a local inside it.")
	d(insADD1cG..tostring(insADD1var08))
	d(insADD1cW.."Global from a function never called => Fail")
	d("9:(global insADD1.wenevercallthis(): ")
	d(insADD1cG..tostring(insADD1var09))
	d(insADD1cW.."Case sensitive references")
	d("10:(INCORRECT: local insADD1var10 testvars):")
	d(insADD1cG..tostring(insADD1var10))
	d("10:(CORRECT: local insadd1var10 testvars):")
	d(insADD1cG..tostring(insadd1var10))
  d("And lastly, here is the clean global variable")
  d(insADD1cG..tostring(insADD1.ADD1varclean))
	d(insADD1cW.."Now try /addtest change to see how Globals can be manipulated from other addons")
end -- this is the end of insADD1.testvars function



-- This is a function that is never called
-- As a result, the global and local variables set inside it will never be made available..
function insADD1.wenevercallthis()
	d("Here is some output you will never see.")
	d("Unless you run /script insADD1.wenevercallthis()")
	d("Course, typing this as d() is pointless when the only way to see the output they provide is from reading the source code, ")
  local insADD1var101 = 42
	insADD1var09 = "Global insADD1var09 from insADD1.wenevercallthis() - Reference insADD1var101 from same function: "..tostring(insADD1var101)
end -- this is the end of insADD1.wenevercallthis function


--------------------------------------------------------------------
---  commandHandler function
--       input: text (Commands)
-- 
-- info:
--    This is the commandHandler, it accepts ANY lenght of input
--    It will separate the input into 'words' separated by a space
--    To make multi-word commands.
--
--    It is referenced to by "SLASH_COMMANDS" below.
--------------------------------------------------------------------
local function commandHandler(text)
 -- put everything in lowercase to let the command handler accept
 -- "LIST" and "list" as the same, for example.
  local input = string.lower(text)
  -- set up some variables
  local com = {}
  local index = 1
  -- separate arguments from 'text' into the 'com' table
  if text~=nil then  -- if text not equals NIL we continue
    -- Here we put each input separated by a space into their own line
    -- in the "com" table.
  	for value in string.gmatch(input,"%w+") do  
  		  com[index] = value
	    	index = index + 1
		end
	end

  -- We go through test cases with the FIRST word the user wrote after "/addtest"
  if com[1]=="1" then -- The user wrote /addtest 1
 		test()
  elseif com[1]=="2" then -- The user wrote /addtest 2
  	test2()
  elseif com[1]=="3" then -- The user wrote /addtest 2
    insADD1optionaltest3()
  elseif com[1]=="4" then -- The user wrote /addtest 2
    insADD1.test4()
  elseif com[1]=="vars" then -- The user wrote /addtest vars
  	testvars()
  elseif com[1]=="table" then -- The user wrote /addtest table
  	testtable()
  elseif com[1]=="testcom" then -- The user wrote /addtest testcom (and hopefully something more)
    d("Here is how the com table looks with your input.")
  	d(com)
  elseif com[1]=="change" then
    d("We call a function in insAddonTest2 addon to change the variables we use in this.")
  	insADD2.change() -- Calls insADD2.change() from insAddonTest2\insAddonTest2.lua
  else  -- if we have no other matches in "com" table, we output the menu.
  	d("No Arguments from me!")
  	d("/addtest 1 : Calls a simple function in insAddonTest1.lua")
  	d("/addtest 2 : Calls a LOCAL function in optional.lua")
    d("/addtest 3 : Calls a GLOBAL function in optional.lua")
    d("/addtest 4 : Calls a GLOBAL more correctly used function in optional.lua")
  	d("/addtest testcom <lots of words> : Shows how input is handled.")
  	d("/addtest vars : Shows variable examples from insAddonTest1.lua")
  	d("/addtest change : Shows how variables can be manpipulated from another addon.")
  	d("/addtest table : Shows how tables can be referenced and manipulated.")
  end
end -- We end our function

--------------------------------------------------------------------
---  Intro function
--       input: nothing
--       output: nothing
-- 
-- info:
--   Send a message to the chat line when the addon is loaded
--------------------------------------------------------------------
local function Intro()
  -- we display some text immediately as the addon is loaded AND the player can see the text
  d("insADD1 is loaded - This is the intro() function reporting in.. type /addtest to begin")
  -- we unregister the event player activated event,  otherwise they'll get this message every
  -- time they zone etc...
  EVENT_MANAGER:UnregisterForEvent("insADD1",EVENT_PLAYER_ACTIVATED)  
end

--------------------------------------------------------------------
---  Initialize function
--       input: eventCode, addOnName
-- 
-- info:
--   Initializes the addon and savedvariables, 
--------------------------------------------------------------------
local function Initialize( eventCode, addOnName )
  -- Just an example to show how this text will not be printed so the user can see it..
  d("Hello, this is insADD1 initialize function. You'll never see this because the player hasn't activated yet.")

-- If the addOnName and the name of our insADD1.name variable do not match
-- We end the initialize function, and the addon will not get loaded further.
     if ( addOnName ~= insADD1.name ) then return end
     
     local insADD1var03 = "Local var insADD1var03 from insADD1.initialize"
     insADD1var04 = "Global var insADD1var04 from insADD1.initialize"
     
     -- We make a table called "insADD1.vars". Tables are enclosed in  {  }
     -- Multiple Keys inside a table are separated by the ","s
     -- We insert a key "key1" with the value "vars-key1-value".
     -- We insert a key "key2" which contains a table.
     -- Inside the table we insert a key "key2key1" with the value "vars-key2key1-value"
     insADD1.vars		= {
					["key1"] = "vars-key1-value",
					["key2"] = {
										["key2key1"] = "vars-key2key1-value", 
										}
     									}  -- we must remember to close the table

    --[[ 
    		insADD1.SV we can reference to from our code. We can then change the value 
        of anything in the savedvariables (Which will be loaded next time the user logs in)
    
       * insADD is the name of savedvariable table.
       * insADD1.vars are the default variables to use when SavedVariables are initialized
       * 1 is the version     
       ** if the version in the users savedvariables table is below this number
       ** the settings will be overwritten by the default settings from insADD1.vars 
       ]]--
		insADD1.SV = ZO_SavedVars:New("insADD_SV" , 1 , nil , insADD1.vars, nil )   
    -- We could also have used the following code to create an ACCOUNTWIDE (Accessible by all characters) savedvariable
    insADD1.SVG = ZO_SavedVars:NewAccountWide("insADD_SV" , 202 , "settings" , insADD1.vars, "Global" )
    -- or even this, for a SavedVariable per character, but 
    insADD1.SVP = ZO_SavedVars:New("insADD_SV" , 202 , "settings", insADD1.vars, "Profile")
    -- Take a look at the /savedvariables/insADD1.lua file to see how the above savedvariables appear.

		-- This tells the game to output a chat message in 4 seconds which should give the game enough time to load the UI.
		zo_callLater(function() d("insADD1 has loaded. /addtest for options: The addonname was: "..addOnName) end, 4000)	 
    -- It is better to use the "intro()" function however. Watch which message comes first..
		
		-- We register a slash command (/addtest) below the function we reference (commandHandler)
		SLASH_COMMANDS["/addtest"] = commandHandler
		
		-- Once the addon is loaded, we no longer need to watch for this.
		EVENT_MANAGER:UnregisterForEvent("insADD1",EVENT_ADD_ON_LOADED)
end  -- end of insADD1.Initialize function

-- We register an event to initialize the addon. 
EVENT_MANAGER:RegisterForEvent("insADD1",EVENT_ADD_ON_LOADED,Initialize)
EVENT_MANAGER:RegisterForEvent("insADD1",EVENT_PLAYER_ACTIVATED, Intro) 